[PATCH] sched: check tgid in is_global_init

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

 



Global init can have sub-threads, so ->pid check is not reliable enough
for is_global_init(), we need to check tgid instead. This has been spotted
by Oleg and a fix was proposed by Richard a long time ago (see the link
below).

Oleg wrote:
:Because is_global_init() is only true for the main thread of /sbin/init.
:
:Just look at oom_unkillable_task(). It tries to not kill init. But, say,
:select_bad_process() can happily find a sub-thread of is_global_init()
:and still kill it.

I recently hit the problem in question; re-sending the patch (to the
best of my knowledge it has never been submitted) with updated function
comment. Credit goes to Oleg and Richard.

Link: https://www.redhat.com/archives/linux-audit/2013-December/msg00086.html
Reported-by: Oleg Nesterov <oleg@xxxxxxxxxx>
Suggested-by: Richard Guy Briggs <rgb@xxxxxxxxxx>
Acked-by: Serge Hallyn <serge.hallyn@xxxxxxxxxxxxx>
Cc: <stable@xxxxxxxxxxxxxxx>
Signed-off-by: Sergey Senozhatsky <sergey.senozhatsky@xxxxxxxxx>
---
 include/linux/sched.h | 5 +++--
 1 file changed, 3 insertions(+), 2 deletions(-)

diff --git a/include/linux/sched.h b/include/linux/sched.h
index daf600e..d33de52 100644
--- a/include/linux/sched.h
+++ b/include/linux/sched.h
@@ -2013,7 +2013,8 @@ static inline int pid_alive(const struct task_struct *p)
 }
 
 /**
  * is_global_init - check if a task structure is init
  * @tsk: Task structure to be checked.
  *
  * Check if a task structure is the first user space task the kernel created.
@@ -2022,7 +2023,7 @@ static inline int pid_alive(const struct task_struct *p)
  */
 static inline int is_global_init(struct task_struct *tsk)
 {
-	return tsk->pid == 1;
+	return task_tgid_nr(tsk) == 1;
 }
 
 extern struct pid *cad_pid;
@@ -2013,7 +2013,8 @@ static inline int pid_alive(const struct task_struct *p)
 
 /**
- * is_global_init - check if a task structure is init
+ * is_global_init - check if a task structure is init. Since init
+ * is free to have sub-threads we need to check tgid.
  * @tsk: Task structure to be checked.
  *
-- 
2.7.0.rc0.20.g4b9ab0e

--
To unsubscribe from this list: send the line "unsubscribe stable" in
the body of a message to majordomo@xxxxxxxxxxxxxxx
More majordomo info at  http://vger.kernel.org/majordomo-info.html



[Index of Archives]     [Linux Kernel]     [Kernel Development Newbies]     [Linux USB Devel]     [Video for Linux]     [Linux Audio Users]     [Yosemite Hiking]     [Linux Kernel]     [Linux SCSI]